/*--------------  FrvBuf.c by B.Mours Sept 14, 2000--------------------------*/

#include "FrvBuf.h"
#include <string.h>

/*------------------------------------------------------------- FrvBufFree --*/
void  FrvBufFree(struct FrvBuf* buffer)
/*---------------------------------------------------------------------------*/
{
  FrVectFree(buffer->output);

  free(buffer);

  return;
}

/*------------------------------------------------------------ FrvBufFeed ---*/
int FrvBufFeed(FrvBuf *buffer, 
               FrVect *vect)
/*---------------------------------------------------------------------------*/
/*  Copy the data in the output structure 
  Returns 0 if more call to FrvBufFeed are needed
          1 if at least one output is ready
	 -1 in case of buffer overflow (the vector is droped)                */
/*---------------------------------------------------------------------------*/
{
 if(buffer == NULL) return(0);
 if(vect == NULL)   return(0);

 buffer->lastCallIsFeed = 1;

          /*----------------------------------- init the object if needed ---*/

 if(buffer->output == NULL)
   {if(FrvBufInit(buffer, vect) == 0) return(NULL);}

          /*----------------- check that the vector size has not changed ----*/

 if(buffer->vectSize != vect->nData*vect->wSize) return(-1);

          /*------------shift existing data if the buffer is full enough ---*/

 if(buffer->gapSize > 0)
   {if(buffer->overSize > 0)
       memmove(buffer->startPtr, 
               buffer->startPtr + buffer->stepSize, buffer->overSize);
    if(buffer->restSize > 0)
    memmove(buffer->startPtr + buffer->overSize, 
            buffer->startPtr + buffer->gapSize, buffer->restSize);

    buffer->restSize += buffer->overSize;}

         /*----------------------- copy the data at the end of the buffer ---*/

 memcpy(buffer->startPtr+buffer->restSize, vect->data, buffer->vectSize);
 buffer->restSize += buffer->vectSize;
 buffer->gapSize = 0;

         /*--------------------------------------- set timing information ---*/

 buffer->output->GTimeS    = vect->GTimeS;
 buffer->output->GTimeN    = vect->GTimeN;
 buffer->output->startX[0] = vect->startX[0];

         /*------------------------------------ is the data resdey to use?---*/

 if(buffer->restSize < buffer->outSize)  return(0);

 return(1);
}

/*
Output buffer management:

    |<-stepSize->|<-overSize->|
    |<-----------outSize----->|
    ###########################---------------------####################
    |<------------gapSize-------------------------->|<----restSize---->|
    |
 startPtr

 if there is no gap, resSize = curSize;
*/
/*------------------------------------------------------------ FrvBufGetNext-*/
int FrvBufGetNext(FrvBuf *buffer)
/*---------------------------------------------------------------------------*/
/*  Prepare the next output buffer 
    Returns: 0 if a call to FrvBufFeed is needed,  
             1 if a vector is available*/
/*---------------------------------------------------------------------------*/
{

 if(buffer == NULL) return(0);

             /*--if we just feed the buffer the output data may be ready ----*/

 if(buffer->lastCallIsFeed == 1) 
   {if(buffer->restSize < buffer->outSize)  return(0);  /*-not enough data---*/

    buffer->lastCallIsFeed = 0; 
    buffer->gapSize  =  buffer->outSize;
    buffer->restSize -= buffer->outSize;
    return(1);}

            /*-----do we have enough data to shift to fill a new segment? ---*/

 if(buffer->restSize < buffer->stepSize) return(0);
 
            /*----------------------------- shift the ovelapping data -------*/

 if(buffer->overSize > 0)
     memmove(buffer->startPtr, 
             buffer->startPtr + buffer->stepSize, buffer->overSize);
   
            /*------------------------------------shift the remaing data ---*/

 memmove(buffer->startPtr + buffer->overSize, 
         buffer->startPtr + buffer->gapSize, buffer->stepSize);

 buffer->gapSize  += buffer->stepSize;
 buffer->restSize -= buffer->stepSize;

 buffer->output->startX[0] +=  buffer->output->dx[0] * buffer->step;

 return(1);
}

/*------------------------------------------------------------- FrvBufInit --*/
int FrvBufInit(FrvBuf *buffer,
               FrVect *vect)
/*---------------------------------------------------------------------------*/
/* Returns 0 in case of problem; 1 otherwise                                 */
/*---------------------------------------------------------------------------*/
{int nData;

 if(vect == NULL) return(0);

             /*---------------------------------- init for output space ----*/

 nData = buffer->nDataOut + vect->nData;

 buffer->output  = FrVectNew1D(vect->name,  vect->type, nData,
                               vect->dx[0], vect->unitX[0], vect->unitY);
 if(buffer->output  == NULL) return(0);

 /*---- the next line is to not tell to the user that there are more
   data in the output buffer -------------------------------------------*/

 buffer->output->nData =  buffer->nDataOut;

 buffer->startPtr = buffer->output->data;
 buffer->restSize = 0;
 buffer->gapSize  = 0;
 buffer->vectSize = vect->nData      *vect->wSize;
 buffer->stepSize = buffer->step     *vect->wSize;
 buffer->outSize  = buffer->nDataOut *vect->wSize;
 buffer->overSize = buffer->outSize - buffer->stepSize;
 
 buffer->lastCallIsFeed = 1;
   
 return(1);
}

/*-------------------------------------------------------------- FrvBufNew --*/
FrvBuf* FrvBufNew(int nDataOut,
                  int step,
                  int outType,
                  int decimate)
/*---------------------------------------------------------------------------*/
/* Create the buffer structure                                               */
/*---------------------------------------------------------------------------*/
{FrvBuf *buffer;

      /*-------- create the buffer structure ----*/

 buffer = (struct FrvBuf *) (calloc(1,sizeof(struct FrvBuf)));
 if(buffer == NULL) return(NULL);
 
 buffer->nDataOut = nDataOut;
 buffer->step     = step;
 buffer->outType  = outType;
 buffer->decimate = decimate;

 return(buffer);
}
