#include <detect.h> 

/*----------------------------------------------------------------------*
 *	Function: VOID Period(p_kont) - Find "floating point"		*
 *						for output		*
 *----------------------------------------------------------------------*
 *	Arguments:							*	
 *			LONG p_kont - samples in the period		*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	The routine Period calculates the period of the detected signal *
 *	 from the value P_kont.  The purpose of this routine is to 	*
 *	 circumvent floating point operations.				*
 *----------------------------------------------------------------------*/

VOID Period(p_kont)
LONG p_kont;
{
	if (con_ptr->sam_sec>0) {
		p_lval = p_kont/con_ptr->sam_sec;
		p_rval = (p_kont % con_ptr->sam_sec) * con_ptr->ns_sam;
	} else {
		p_lval = p_kont * (con_ptr->ns_sam / 1000000000);
		p_rval = 0;
		/* printf("p_kont=%d,ns_sam=%d,p_lval=%d\n",
			p_kont,con_ptr->ns_sam,p_lval); */
 	}
	return;
}		

#ifdef JNMCOMMENT
/*************************************************************************

			EXPLANATION OF VARIABLES

	ns_sam -- microseconds between successive samples of the digital
		    data.  See Pass_one.
	
	p_kont	-- the average number of samples per cycle in the first four 
		    cycles of the declared signal.  See Onsetq.

	p_lval 	-- the integer part of the period. Seconds.

	p_rval	-- the remainder (decimal fraction) of the period
		    expressed in nanoseconds.

	sam_sec -- samples per second of the digital data.
		    See E_detect, Pass_one.

************************************************************************/
#endif
