// $Id: hoftMon.hh,v 1.3 2008/11/04 18:02:50 jordi Exp $

#ifndef HOFTMON_HH
#define HOFTMON_HH

#include "DVecType.hh"
#include "DatEnv.hh"
#include "FrWriter.hh"
#include "MonServer.hh"
#include "FixedLenTS.hh"
#include "Trend.hh"
#include <string>

extern "C" {
#include <lal/Calibration.h>
}


/*
 * @name    hoftMon
 * @memo    Compute calibrated h(t)
 * @author  Jordi Burguet-Castell, Shourov K. Chatterji, Xavier Siemens
 * @version 0.92
 */
class hoftMon : public DatEnv, MonServer {
public:
    hoftMon(int argc, const char *argv[]);
    ~hoftMon(void);
    void ProcessData(void);      // process 1 data stride
    void Attention(void);        // handle message signal

private:
    double      mStep;           // time stride (duration of the data chunk)
    FrWriter    mFrameWriter;    // frame writer
    int         mVerboseLevel;   // verbosity
    int         mIteration;      // in which iteration we are

    StrainIn    mInputData;
    StrainOut   mOutputData;
    
    static const char* mChanNames[];

    int  get_channels(const TSeries* dmt_chans[]);
    int  initialize_input_data(int argc, const char *argv[]);
    int  allocate_data(const TSeries* dmt_chans[]);
    int  free_data();
    int  destroy_initialization();
    void read_data(const TSeries* dmt_chans[]);
    int  write_frames(const TSeries* dmt_chans[]);
};


// Name of the channels we will read.
const char* hoftMon::mChanNames[] = {
  "H2:LSC-AS_Q",
  "H2:LSC-DARM_CTRL",
  "H2:LSC-DARM_ERR",
  "H2:LSC-DARM_CTRL_EXC_DAQ"
};


#endif     //  HOFTMON_HH
