#include "sigma.hh"										// Program writted jointly by Akash Kansagra of MIT and Ryan Williams of LSU
#ifndef __CINT__
#include <iostream>								//							*********************
#include <iomanip>									//							  ********************
#include <fstream>									//								 ****				   **
#include <stdlib.h>									//							 		****
#include <stdio.h>									//							  		  ****
#include <time.h>										//										 ****
#include <math.h>										//									  ****
#include "Histogram1.hh"							//									****
#include "TSeries.hh"								//								 *****				**
#include "FSeries.hh"								//							  ********************
#include "FSpectrum.hh"								//							*********************
#include "Dacc.hh"

using namespace std;

const char *stoc(string stoc_s) {
   char *stoc_c;
   stoc_c=new char [stoc_s.length ()+1];
   stoc_s.copy(stoc_c, string::npos);
   stoc_c[stoc_s.length()]='\0';
   return stoc_c;
};
string itos(long x, unsigned int n=10) {
   unsigned int p; string str="";
   for (p=0; p<n; p++) { str+="0"; }
   for (p=0; p<n; p++) {
       str [str.length ()-p-1]=(char)(x%10+'0');
       x/=10; }
   return str;
};
void displayHelp() {
	cout << "SIGMA: SImple Graphical Monitoring Applet -> USAGE: ./sigma <options>" << endl;
	cout << "OPTIONS (relevant input supplied after tag with no intervening whitespace):" << endl;
	cout << " -c  Sets channel. Channel name follows -c tag. Default H0:PEM-LVEA_SEISZ." << endl;
	cout << " -t  Sets time interval. Observation interval in seconds follows tag. Default 1." << endl;
	cout << " -d  Toggles display. If selected, a X window will be displayed with the" << endl << "       selected plots. Default ON." << endl;
	cout << " -s  Toggles save. If selected, a .log file containing the data will be written" << endl << "       to disk.  Default OFF." << endl;
	cout << " -x  Enables image save. Format (ex. gif, ps, ai) follows the tag. Default OFF." << endl;
	cout << " -T  Toggles time series plot. If either the -d or -x options are set, data in" << endl << "       the time domain will be displayed.  Default OFF." << endl;
	cout << " -F  Toggles frequency spectrum plot. Same behavior as -T, but one can" << endl << "       optionally specify a plot style (0=lin-lin, 1=log-log, 2=lin-log," << endl << "       3=log-lin) after -F." << endl;
	cout << " -H  Toggles histogram plot. Displayed in same space as -G and -R. Default OFF." << endl;
	cout << " -G  Toggles gaussian fit. Displayed in same space as -H and -R. Default OFF." << endl;
	cout << " -R  Toggles residual plot. Displayed in same space as -H and -R. Default OFF." << endl;
	cout << " -b  Sets # bins. Number of bins in the histogram follows the tag. Default 100." << endl;
	cout << " -l  Toggles log axis on histogram/fit/residual. If -R is selected, it will be" << endl << "       toggled off. Bins with no contents will be thrown out. Default OFF." << endl;
	cout << " -h  Displays help. Setting this option displays this help and exits the" << endl << "       application. Equivalent to \'--help\'." << endl;
	cout << "EXAMPLE: -cH0:PEM-COIL_MAGX tells SIGMA to monitor the H0:PEM-COIL_MAGX channel." << endl;
};
//--------------------------------------  Generate the main routine.
EXECDAT(DatTemplate)
#endif               // !def(__CINT__)

time_t ct;
const long numSamples=33000;
double g[numSamples], timeInterval=1.0; // data, duration
int toggleT=0, toggleH=0, toggleF=0, toggleG=0, toggleR=0, toggleS=0, toggleL=0, toggleD=1, axesScale=0, nbin=100;
string Ch("H0:PEM-LVEA_SEISZ"), outputMode="";

//--------------------------------------  Skeleton object constructor.
DatTemplate::DatTemplate(int argc, const char *argv[]) 
  : DatEnv(argc, argv), MaxFrame(999999)
{
	string str;
	for (int x=1; x<(int)argc; x++) {
		str=string(argv[x]);
		if (str.substr(0,2) == "-c") // channel name
			{	Ch=str.substr(2, str.length()-2);	}
		if (str.substr(0,2) == "-t") // time interval
			{	timeInterval=strtod( stoc(str.substr(2, str.length()-2)),NULL );	}
		if (str.substr(0,2) == "-s") // save leardata file
			{	toggleS=1-toggleS;	}
		if (str.substr(0,2) == "-x") // output graphics file encoding method (prevent from being X)
			{	if (str.length() > 2) {	outputMode=str.substr(2, str.length()-2); }	}
		if (str.substr(0,2) == "-d") // toggle X display
			{	toggleD=1-toggleD;	}
		if (str.substr(0,2) == "-b") // set histogram bins
			{	nbin=(int)strtod(stoc(str.substr(2, str.length()-2)),NULL);	}
		if (str.substr(0,2) == "-T") // toggle time series
			{	toggleT=1-toggleT;	}
		if (str.substr(0,2) == "-H") // toggle histogram
			{	toggleH=1-toggleH;	}
		if (str.substr(0,2) == "-l") // log axis for distribution plot
			{  toggleL=1-toggleL;   }
		if (str.substr(0,2) == "-R") // toggle residual
			{	toggleR=1-toggleR;	}
		if (str.substr(0,2) == "-G") // toggle gaussian
			{	toggleG=1-toggleG;	}
		if (str.substr(0,2) == "-F") // toggle frequency spectrum
			{	toggleF=1-toggleF; str+="0";
				axesScale=(int)strtod( stoc(str.substr(2,1)),NULL );	}
		if (str.substr(0,2) == "-h" || str == "--help") { displayHelp(); exit(0); }
	}
	// verify correctness of arguments
	if (timeInterval<0.1) { cout << "Entered time falls below minimum. Defaulting to 0.1 s."; getDacc().setStride(0.1); }
	else if (timeInterval>512) { cout << "Entered time exceeds maximum. Defaulting to 512 s." << endl; getDacc().setStride(512); }
	else { getDacc().setStride(timeInterval); }
	if (Ch.length()<10) { cout << "Inappropriate channel entered.  Defaulting to H0:PEM-LVEA_SEISZ." << endl; Ch="H0:PEM-LVEA_SEISZ"; }
	if (nbin<1) { cout << "Inappropriate number of bins entered.  Defaulting to 100." << endl; nbin=100; }
	if (toggleL==1 && toggleR==1) { cout << "Logarithmic axis incompatible with residual.  Toggling residual off." << endl; toggleR=0; }
	if (outputMode=="X") { outputMode=""; }
	getDacc().addChannel(stoc(Ch));
}

//--------------------------------------  Skeleton object destructor.
DatTemplate::~DatTemplate() 
	{	cout << "DatTemplate is finished" << endl;	}

void DatTemplate::ProcessData (void)
{
   TSeries *tx=getDacc().refData(stoc(Ch));			// gather time series data
   FSeries Frequency(*tx);
   FSpectrum fx(Frequency);							// create FSpectrum
   long tsn = tx->getNSample();					// tsn = number of time samples
   double tsd = tx->getInterval();					// tsd = tseries duration
   long fsn = fx.getNStep();						// fsn = number of frequency samples
   double fsd = fx.getFStep();						// fsd = frequency sample spacing
   double min = tx->getMinimum(), max = tx->getMaximum();
   tx->getData(tsn, g);								// store tx data in g[]
	Histogram1 hx("Distribution",nbin,min,max,"Value","Number"); // create histogram
   hx.FillN((int)tsn, g);
   double mean = hx.GetMean(), stdDev = hx.GetSdev(), sum=0, temp;
	// writing files
	ofstream fileT(".___fileT"), fileH(".___fileH"), fileF(".___fileF"), fileG(".___fileG"), fileR(".___fileR"), fileD(".___fileD");
	fileD << 1 << "\t\t" << 1 << endl << 2 << "\t\t" << 1 << endl << 3 << "\t\t" << 1 << endl;
	for (long j=0; j<tsn; j++) { fileT << j*tsd/(tsn) << "\t\t" << g[j] << endl; }
	for (long j=1; j<fsn; j++) { fileF << j*fsd << "\t\t" << fx(j*fsd) << endl; } // skips f=0 (just a constant, and screws up log plot)
	fileH << min << "\t\t" << 0 << endl;
	for (int j=0; j<nbin; j++) {
		fileH << hx.GetBinCenter(j) << "\t\t" << hx.GetBinContent(j) << endl;
		sum+= ( hx.GetBinContent(j) * ( hx.GetBinLowEdge(j+1)-hx.GetBinLowEdge(j) ) ); }
	fileH << max << "\t\t" << 0;
	sum/=(sqrt(2*3.1415926535898)*stdDev);
	for (int j=0; j<nbin; j++) {
		temp = sum*exp( -pow(hx.GetBinCenter(j)-mean,2)/(2*stdDev*stdDev) );
		fileG << hx.GetBinCenter(j) << "\t\t" << temp << endl;
		fileR << hx.GetBinCenter(j) << "\t\t" << hx.GetBinContent(j)-temp << endl;	}
	// files have all been written
	string axesString="", histAxis="", command="", graphLabel = "\""+Ch+" + "+  itos((long)((tx->getStartTime()).totalS()))   +"\"";
	if (axesScale==1) { axesString="-l y -l x"; }
	else if (axesScale==2) { axesString="-l y"; }
	else if (axesScale==3) { axesString="-l x"; }
	if (toggleL==1) { histAxis="-l y"; }
	// creating system commands
	if (toggleT==1 && (toggleH==1 || toggleG==1 || toggleR==1) && toggleF==1) {	// display all three plots
		command="--bitmap-size 800x600 -F HersheySans -f 0.05 --title-font-name HersheySans --title-font-size 0.06 -L \"Signal\" --reposition 0 .5 1 -w 0.5 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \"Time [s]\" -Y \" \" -C -m 1 -g 1 .___fileT";
		command+=" --reposition 0 .5 1 -t -N x -w 0.25 -r 0.65 -h 0.38 -u 0.06 -f 0.076 --blankout 1 -X \"\" -Y \"Count\" -g 2 "+histAxis+" -L Distribution"; // 0.0912
		if (toggleH==1) { command+=" -m 1 .___fileH"; }
		if (toggleR==1) { command+=" -m 2 .___fileR"; }
		command+=" -C";
		if (toggleG==1) { command+=" -m 1 .___fileG"; }
		command+=" --reposition 0 0 1 -t -w 0.8 -r 0.1 -h 0.38 -u 0.06 -f 0.05 -N x --blankout 1 -X \"Frequency [Hz]\" -Y \"Power [a.u.]\" -g 3 -L \"Power Spectrum\" "+histAxis+" "+axesString+" -C -m 1 .___fileF";
		command+=" --reposition 0.93 0 1 -h .7 -u .15 -w .7 -r .15 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==1 && (toggleH==0 && toggleG==0 && toggleR==0) && toggleF==0) {	// display TSeries only
		command="--bitmap-size 800x600 -F HersheySans -f 0.035 --title-font-name HersheySans --title-font-size 0.04 -g 1 -w 0.7 -h 0.8 -r 0.15 -u 0.1 --blankout 1 -X \"Time [s]\" -L \"Signal\" -C -m 1 .___fileT";
		command+=" --reposition 0.93 0 1 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==0 && (toggleH==0 && toggleG==0 && toggleR==0) && toggleF==1) {	// display FSeries only
		command="--bitmap-size 800x600 -F HersheySans -f 0.035 "+axesString+" --title-font-name HersheySans --title-font-size 0.04 -g 3 -w 0.7 -h 0.8 -r 0.15 -u 0.1 --blankout 1 -X \"Frequency [Hz]\" -Y \"Power [a.u.]\" -L \"Power Spectrum\" -C -m 1 .___fileF";
		command+=" --reposition 0.93 0 1 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==0 && (toggleH==1 || toggleG==1 || toggleR==1) && toggleF==0) {	// display Histogram only
		command="--bitmap-size 800x600 -F HersheySans -f 0.035 --title-font-name HersheySans --title-font-size 0.04 -w 0.7 -h 0.8 -r 0.15 -u 0.1 --blankout 1 -X \" \" -Y \"Count\" "+histAxis+" -L \"Distribution\"";
		if (toggleH==1) { command+=" -C -m 1 .___fileH"; }
		if (toggleR==1) {
			if (toggleH==1) { command+=" -m 2 .___fileR"; }
			else { command+=" -C -m 2 .___fileR"; } }
		if (toggleG==1) {
			if (toggleH==1 || toggleG==1) { command+=" -C -m 2 .___fileG"; }
			else { command+=" -m 2 .___fileG"; } }
		command+=" --reposition 0.93 0 1 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==1 && !(toggleH==1 || toggleG==1 || toggleR==1) && toggleF==1) {	// display TSeries and FSeries
		command="--bitmap-size 800x600 -F HersheySans -f 0.05 --title-font-name HersheySans --title-font-size 0.06 -L \"Signal\" --reposition 0 .5 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \"Time [s]\" -Y \" \" -C -m 1 -g 1 .___fileT";
		command+=" --reposition 0 0 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \"Frequency [Hz]\" -Y \"Power [a.u.]\" -L \"Power Spectrum\" "+axesString+" -m 1 -g 3 .___fileF";
		command+=" --reposition 0.93 0 1 -h .7 -u .15 -w .7 -r .15 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==1 && (toggleH==1 || toggleG==1 || toggleR==1) && toggleF==0) {	// display TSeries and Histogram
		command="--bitmap-size 800x600 -F HersheySans -f 0.05 --title-font-name HersheySans --title-font-size 0.06 -L \"Signal\" --reposition 0 .5 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \"Time [s]\" -Y \" \" -C -m 1 -g 1 .___fileT";
		command+=" --reposition 0 0 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \" \" -Y \"Count\" -g 2 "+histAxis+" -L \"Distribution\"";
		if (toggleH==1) { command+=" -m 1 .___fileH"; }
		if (toggleR==1) { command+=" -m 2 .___fileR"; }
		if (toggleG==1) { command+=" -C -m 2 .___fileG"; }
		command+=" --reposition 0.93 0 1 -h .7 -u .15 -w .7 -r .15 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==0 && (toggleH==1 || toggleG==1 || toggleR==1) && toggleF==1) {	// display FSeries and Histogram
		command="--bitmap-size 800x600 -F HersheySans -f 0.05 --title-font-name HersheySans --title-font-size 0.06 --reposition 0 0 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \" \" -Y \"Count\" "+histAxis+" -L \"Distribution\" -C";
		if (toggleH==1) { command+=" -m 1 .___fileH"; }
		if (toggleR==1) { command+=" -m 2 .___fileR"; }
		command+=" -C";
		if (toggleG==1) { command+=" -m 2 .___fileG"; }
		command+=" --reposition 0 .5 1 -w 0.8 -r 0.1 -h 0.38 -u 0.06 --blankout 1 -X \"Frequency [Hz]\" -Y \"Power [a.u.]\" -g 3 "+histAxis+" -L \"Power Spectrum\" "+axesString+" -C -m 1 .___fileF";
		command+=" --reposition 0.93 0 1 -h .7 -u .15 -w .7 -r .15 -F HersheySans -f 0.05 --g 4 --blankout 1 -Y "+graphLabel+" .___fileD"; } // magician's trick to label entire window
	else if (toggleT==0 && toggleF==0 && toggleH==0 && toggleR==0 && toggleG==0 && (toggleD==1 || outputMode!=""))	{ // if no plots selected and some output method is selected, do not allow
		cout << "No plots selected.  Cancelling request to draw." << endl; toggleD=0;	outputMode="";	}
	// process command + save file + clean up
	string outputFilename=Ch+"-"+itos((long)((tx->getStartTime()).totalS()));
	if (toggleD==1) {	system(stoc("graph -T X "+command)); } // draw to X window
	if (outputMode!="") { system(stoc("graph -T "+outputMode+" "+command+" > "+outputFilename+"."+outputMode));	} // draw to image file
	if (toggleS==1) {	// save data to text file
		ofstream file(stoc(outputFilename+".log"));
		file << "Time [s]\t\t" << Ch << endl;
		for (long x=0; x<tsn; x++) { file << setiosflags(ios::left) << setw(10) << x*tsd/tsn << "\t\t" << g[x] << endl; } }
	system("rm .___fileT .___fileH .___fileF .___fileG .___fileR .___fileD");
   exit(0);
}
