//
//    TrigProc implementation functions.
//
#include "PConfig.h"
#if defined(sun) && !defined(__EXTENSIONS__)
#define __EXTENSIONS__
#endif
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <strings.h>
#include "TrigProc.hh"
#include "Time.hh"
#include <iostream>

using namespace trig;


//======================================  Create a trigger process identifier.
TrigProc::TrigProc(const TrigProc& x) 
  : mCreatorDB(x.mCreatorDB), mProgram(x.mProgram), mVersion(x.mVersion),
    mSource(x.mSource), mTime(x.mTime), mAuthor(x.mAuthor), 
    mComment(x.mComment), mIsOnline(x.mIsOnline), mNode(x.mNode),
    mProcID(x.mProcID), mStartTime(x.mStartTime), mEndTime(x.mEndTime),
    mCvsID(x.mCvsID), mProcessID(x.mProcessID), mParamSet(x.mParamSet), 
    mIfos(x.mIfos)
{}

TrigProc::TrigProc(const char* cvsid, const char* title) 
  : mCreatorDB(0)
{
    int yy, mo, dd, hh, mm, ss;
    struct tm strtime;
    time_t stime;

    //----------------------------------  Parse out the cvs info
    char Source[128], Version[8], Author[64];
    Source[0]  = 0;
    Version[0] = 0;
    Author[0]  = 0;
    if (cvsid) {
        mCvsID = cvsid;
        sscanf(cvsid, "$Header: %s %s %4d/%2d/%2d %2d:%2d:%2d %s ", 
	       Source, Version, &yy, &mo, &dd, &hh, &mm, &ss, Author);
	strtime.tm_sec   = ss;
	strtime.tm_min   = mm;
	strtime.tm_hour  = hh;
	strtime.tm_mday  = dd;
	strtime.tm_mon   = mo-1;
	strtime.tm_year  = yy-1900;
	strtime.tm_wday  = 0;
	strtime.tm_yday  = 0;
	strtime.tm_isdst = 0;
	// printf("CVS header: %s\n", cvsid);
	// printf("CVS update time= %i/%i/%i %i:%i:%i\n", 
	//        strtime.tm_mon+1, strtime.tm_mday, strtime.tm_year,
	//        strtime.tm_hour,  strtime.tm_min,  strtime.tm_sec);
	stime = mktime(&strtime);
	mTime = fromUTC(stime).getS();
	mSource  = Source;
	mVersion = Version;
	mAuthor  = Author;
    }

    //----------------------------------  Get the descriptive title
    if (title) mComment = title;

    //----------------------------------  Program name is the executable
#if !defined (linux) && !defined (P__WIN32)
    const char* Xname = getexecname();
#elif defined (P__WIN32)
#define MAXXNAME 512
    char Xname[MAXXNAME], Path[MAXXNAME];
    sprintf(Path, "/proc/%i/exename", getpid());
    int rc = readlink(Path, Xname, MAXXNAME);
    if (rc > 0) Xname[rc] = 0;
    else        std::cout << "Process name not available!" << std::endl;
#else
#define MAXXNAME 512
    char Xname[MAXXNAME], Path[MAXXNAME];
    sprintf(Path, "/proc/%i/exe", getpid());
    int rc = readlink(Path, Xname, MAXXNAME);
    if (rc > 0) Xname[rc] = 0;
    else        std::cout << "Process name not available!" << std::endl;
#endif
    unsigned int inx=0, len=0;
    if (Xname) {
        for (int i=0 ; Xname[i] ; i++) {
	    if (Xname[i] == '/') {
	        inx = i+1;
		len = 0;
	    } else {
	        len++;
	    }
	}
    }
    if (len) mProgram = std::string(Xname, inx, len);

    //----------------------------------  Set the rest of the parameters.
    mStartTime = Now().getS();
    mEndTime   = 0;
    mIsOnline  = true;
    mProcID    = getpid();
    char Hname[128];
    gethostname(Hname, sizeof(Hname));
    mNode = Hname;
    const char* IFOs = getenv("DMTIFOS");
    if (IFOs)  mIfos = IFOs;
}

//======================================  Construct from everything
TrigProc::TrigProc(const char* Source, const char* Version, 
			 gps_t Start, const char* Host, pid_t PID, 
			 bool Online, const char* IFOs, const char* Author, 
			 gps_t Mod, const char* Title, const char* Exec, 
			 gps_t End) 
  : mCreatorDB(0), mVersion(Version), mSource(Source), mTime(Mod),
    mIsOnline(Online), mProcID(PID), mStartTime(Start), mEndTime(End)
{

    //----------------------------------  Get the descriptive title
    if (Title)  mComment = Title;
    if (Author) mAuthor  = Author;
    if (Exec)   mProgram = Exec;
    if (Host)   mNode    = Host;
    if (IFOs)   mIfos    = IFOs;
}

//======================================  Destroy a trigger process.
TrigProc::~TrigProc() {
}

//======================================  Get the author's name.
const char*
TrigProc::getAuthor() const {
    return mAuthor.c_str();
}

//======================================  Get the end time.
TrigProc::gps_t 
TrigProc::getEndTime(void) const {
    return mEndTime;
}

//======================================  Get the interferometer list.
const char* 
TrigProc::getIFOs(void) const {
    return mIfos.c_str();
}

//======================================  Get the source version time.
TrigProc::gps_t 
TrigProc::getModTime(void) const {
    return mTime;
}

//======================================  Get the process name.
const char*
TrigProc::getName() const {
    return mProgram.c_str();
}

//======================================  Get the node name
const char* 
TrigProc::getNode(void) const {
    return mNode.c_str();
}

//======================================  Get the parameter list
const char* 
TrigProc::getParamSet(void) const {
    return mParamSet.c_str();
}

//======================================  Get the database id
const char* 
TrigProc::getProcessID(void) const {
    return mProcessID.c_str();
}

//======================================  Get the process ID.
pid_t
TrigProc::getProcID() const {
    return mProcID;
}

//======================================  Get the source code location
const char* 
TrigProc::getSource(void) const {
    return mSource.c_str();
}

//======================================  Get the Start time
TrigProc::gps_t 
TrigProc::getStartTime(void) const {
    return mStartTime;
}

//======================================  Get the process title.
const char*
TrigProc::getTitle() const {
    return mComment.c_str();
}

//======================================  Get the source version
const char* 
TrigProc::getVersion(void) const {
    return mVersion.c_str();
}

//======================================  Test if process is online.
bool 
TrigProc::isOnline(void) const {
    return mIsOnline;
}

//======================================  Overloaded operators.
TrigProc& 
TrigProc::operator=(const TrigProc& x) {
    mCreatorDB = x.mCreatorDB;
    mProgram   = x.mProgram;
    mVersion   = x.mVersion;
    mSource    = x.mSource;
    mTime      = x.mTime;
    mAuthor    = x.mAuthor;
    mComment   = x.mComment;
    mIsOnline  = x.mIsOnline;
    mNode      = x.mNode;
    mProcID    = x.mProcID;
    mStartTime = x.mStartTime;
    mEndTime   = x.mEndTime;
    mProcessID = x.mProcessID;
    mParamSet  = x.mParamSet;
    mIfos      = x.mIfos;
    return *this;
}

bool 
TrigProc::operator==(const TrigProc& x) const {
    return  (mProcID == x.mProcID && mStartTime == x.mStartTime &&
	     mNode   == x.mNode   && mProgram   == x.mProgram);
}

//=====================================  Set the process comment string
void 
TrigProc::setComment(const std::string& comment) {
    mComment = comment;
}

//=====================================  Set the cvs ID
void
TrigProc::setCvsID(const std::string& cvs) {
    mCvsID = cvs;
}

//=====================================  Set the process end time
void 
TrigProc::setEndTime(gps_t tEnd) {
    mEndTime = tEnd;
}

//=====================================  Set the process IFO list
void
TrigProc::setIFOs(const std::string& ifos) {
    mIfos = ifos;
}

//=====================================  Set the process database ID
void
TrigProc::setProcessID(const std::string& ID) {
    mProcessID = ID;
}
