
dnl -*- m4 -*-
dnl----------------------------------------------------------------------
dnl This is a common set of macros to be used in the configuration of
dnl	the GDS software
dnl----------------------------------------------------------------------

dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_GDS_PROG_CC],
[ AC_PROG_CC
if test "${GCC}" = "yes"
then
  touch ,t.c
  ac_dmt_default_cc_includes="`${CC} -v ,t.c 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.c
fi
GDS_CC_FLAVOR="generic"
case $CC in
gcc|*/gcc) GDS_CC_FLAVOR="gnu";;
cc|*/cc)
    case "`$CC -V 2>&1`" in
    *Sun\ C\ *)
      GDS_CC_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AM_CONDITIONAL([GDS_CC_SUNPRO],[test x$GDS_CC_FLAVOR = xSunPRO])
])
 
dnl ---------------------------------------------------------------------
 
AC_DEFUN([AC_GDS_PROG_CXX],
[ AC_PROG_CXX
if test "${GXX}" = "yes"
then
  touch ,t.cc
  ac_dmt_default_cxx_includes="`${CXX} -v ,t.cc 2>&1 | \
        sed -n -e '/\#include/,/End of search list\./p' | \
        egrep '^ '`"
  rm ,t.cc
fi
GDS_CXX_FLAVOR="generic"
case $CXX in
gcc|*/gcc|g++|*/g++) GDS_CXX_FLAVOR="gnu";;
CC|*/CC)
    case "`$CXX -V 2>&1`" in
    *Sun\ C++*)
      GDS_CXX_FLAVOR="SunPRO"
      ;;
    esac
    ;;
esac
AC_SYS_LARGEFILE
AM_CONDITIONAL([GDS_CXX_SUNPRO],[test x$GDS_CXX_FLAVOR = xSunPRO])
])
 

dnl======================================================================
dnl GDS_CHECK_ROOT
dnl   Check if ROOT is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_ROOT],
[
rootcflags=
rootlibs=
rootglibs=
AC_MSG_CHECKING(["for ROOT"])
if test "x$rootdir" = "x"
then
  rootdir=$ROOTSYS
fi
if test -x $rootdir/bin/root-config
then
  rootbin=$rootdir/bin
  rootcflags=`$rootbin/root-config --cflags`
  rootlibs=`$rootbin/root-config --libs`
  rootglibs=`$rootbin/root-config --glibs`
  rootlib=`echo $rootlibs | sed -e 's|^-L\([[^ ]]*\).*$|\1|'`
  rootinc=`echo $rootcflags | sed -e 's|^-I\([[^ ]]*\).*$|\1|'`
  rootcint="$rootbin/rootcint"
  rootbaselibs="-L`$rootbin/root-config --libdir` -lCint -lCore `$rootbin/root-config --auxlibs`"
  LD_LIBRARY_PATH="$rootlib:$LD_LIBRARY_PATH"
else
  AC_MSG_RESULT([no])
  echo "ROOT must be on the system - Aborting"
  exit 1
fi
AC_MSG_RESULT([${rootdir}])
AC_SUBST(rootdir)
AC_SUBST(rootbin)
AC_SUBST(rootlib)
AC_SUBST(rootinc)
AC_SUBST(rootcflags)
AC_SUBST(rootlibs)
AC_SUBST(rootglibs)
AC_SUBST(rootbaselibs)
AC_SUBST(rootcint)
AC_SUBST(LD_LIBRARY_PATH)
])

dnl======================================================================
dnl GDS_CHECK_EXPAT
dnl   Check if EXPAT is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_EXPAT],
[
have_expat="no"
AC_CHECK_LIB(xmlparse, XML_Parse, have_expat="yes", have_expat="no")
AM_CONDITIONAL([require_expat], [test "$have_expat" = "no" ])
if test "$have_expat" = "yes"; then
  pkgf=/usr
  pkgl=$pkgf/lib
  for d in `echo "$LD_LIBRARY_PATH" | sed -e "s/:/ /g"` ; do
    if test -f "$d/libxmlparse.so.0" ; then
      pkgl=`echo $d | sed -e 's,/$,,g'`
      pkgf=`echo $pkgl | sed -e 's,/lib\(64\)*$,,'`
      echo $pkgf
      break
    fi
  done
  expatcflags="-I$pkgf/include"
  expatlibs="-L$pkgl -lxmlparse"
else
  expatcflags="-I$PWD/External/include"
  expatlibs="-L$PWD/External/lib -lxml"
fi
AC_SUBST(expatcflags)
AC_SUBST(expatlibs)
])

dnl======================================================================
dnl GDS_CHECK_FFTW3
dnl   Check if FFTW3 is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FFTW3],
[
AC_CHECK_LIB(fftw3f, fftwf_execute_dft, have_fftw3f="yes", have_fftw3f="no")
AC_CHECK_LIB(fftw3,  fftw_execute_dft,  have_fftw3d="yes", have_fftw3d="no")
AM_CONDITIONAL([require_fftw3], [test "$have_fftw3f" = "no" -a "$have_fftw3d" = "no" ])
if test "$have_fftw3f" = "yes"; then
  pkgf=/usr
  pkgl=$pkgf/lib
  for d in `echo "$LD_LIBRARY_PATH" | sed -e "s/:/ /g"` ; do
    if test -f "$d/libfftw3f.so" -o -f "$d/libfftw3f.a" ; then
      pkgl=`echo $d | sed -e 's,/$,,g'`
      pkgf=`echo $pkgl | sed -e 's,/lib\(64\)*$,,'`
      echo $pkgf
      break
    fi
  done
  fftwcflags="-I$pkgf/include -DFFTW3"
  fftwlibs="-L$pkgl -lfftw3f"
  if test "$have_fftw3d" = "yes"; then
    fftwcflags="$fftwcflags -DFFT_DOUBLE"
    fftwlibs="$fftwlibs -lfftw3"
  fi
else
  fftwcflags="-I$PWD/External/include -DFFTW3 -DFFT_DOUBLE"
  fftwlibs="-L$PWD/External/lib -lfftw3f -lfftw3"
fi
AC_SUBST(fftwcflags)
AC_SUBST(fftwlibs)
])

dnl======================================================================
dnl GDS_FIND_PKG
dnl Check if soecified package is installed
dnl======================================================================

AC_DEFUN([AC_GDS_FIND_PKG],
[
symname="$1"
pkgname="$2"
])

dnl======================================================================
dnl GDS_CHECK_ZLIB
dnl   Check if ZLIB is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_ZLIB],
[
found_zutil=no
zlibcflags=
zliblibs="-lz"
have_libz=no
AC_CHECK_LIB([z], inflate, have_libz="yes", have_libz="no")
libp=`echo $LD_LIBRARY_PATH | sed -e s,::,:,g -e 's,/lib\(64\)*\:, ,g' -e 's,/lib\(64\)*$,,g'`
for p in $libp /usr/local /usr ; do
  if test -f $p/include/zutil.h ; then
    zlibcflags="-I$p/include"
    zliblibs="-L$p/lib -lz"
    found_zutil=yes
    break
  elif test -f $p/include/zlib.h ; then
    zlibcflags="-I$p/include"
  fi
done
AM_CONDITIONAL([have_zutil], [test "$found_zutil" = "yes"])
AC_SUBST(zlibcflags)
AC_SUBST(zliblibs)
if test $have_libz = "no" ; then
  AC_MSG_ERROR([libz not found])
fi
])

dnl======================================================================
dnl GDS_CHECK_FRAMECPP
dnl   Check if framecpp is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FRAMECPP],
[
have_framecpp=
PKG_CHECK_MODULES(FRAMECPP, framecpp, have_framecpp="yes", have_framecpp="no")
if test "${have_framecpp}" = "yes"
then
framecpplibs="$FRAMECPP_LIBS"
framecppcflags="$FRAMECPP_CFLAGS"
fi
if test "${have_framecpp}" = "no"
then
framecppcflags="-I../../External/include"
framecpplibs="-L../../External/lib -lframecpp -lframecppcmn -lframecpp6 -lframecpp3_4_5 -lgeneral"
fi
AM_CONDITIONAL([require_framecpp], [test "$have_framecpp" = no])
AC_SUBST(framecpplibs)
AC_SUBST(framecppcflags)
])


dnl======================================================================
dnl GDS_CHECK_LAL
dnl   Check if lal is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_LAL],
[
lal_pkgok=
PKG_CHECK_MODULES(LAL, [lal lalframe lalsupport], lal_pkgok="yes", lal_pkgok="no")
if test "${lal_pkgok}" = "yes"
then
lallibs="$LAL_LIBS"
lalcflags="$LAL_CFLAGS"
else
lallibs=
lalcflags=
fi
AM_CONDITIONAL([have_lal], [test "$lal_pkgok" = yes])
AC_SUBST(lallibs)
AC_SUBST(lalcflags)
])


dnl======================================================================
dnl GDS_CHECK_FR
dnl   Check if FR is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_FR],
[
have_fr=no
PKG_CHECK_MODULES(FRAME, libframe, have_fr="yes", have_fr="no")
if test "${have_fr}" = "yes"
then
frlibs="$FRAME_LIBS"
frcflags="$FRAME_CFLAGS"
fi
if test "${have_fr}" = "no"
then
frlibs="-lFrame"
frcflags=""
case `uname -m` in
   "i386" | "i486" | "i586" | "i686")
	frcflags=""
	;;
   *)
	frcflags="-DFR_LONG_LONG"
	;;
esac
fi
AM_CONDITIONAL([require_fr], [test "$have_fr" = no])
AC_SUBST(frlibs)
AC_SUBST(frcflags)
])

dnl======================================================================
dnl GDS_CHECK_METAIO
dnl   Check if METAIO is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_METAIO],
[
have_metaio=
PKG_CHECK_MODULES(METAIO, libmetaio, have_metaio="yes", have_metaio="no")
if test "${have_metaio}" = "yes"
then
metaiovsn=`pkg-config --modversion libmetaio | sed -e 's,\.,0,g'`
metaiolibs="$METAIO_LIBS"
metaiocflags="$METAIO_CFLAGS -DMETAIO_VSN=$metaiovsn"
fi
if test "${have_metaio}" = "no"
then
metaiolibs="-lmetaio"
metaiocflags=""
fi
AM_CONDITIONAL([require_metaio], [test "$have_metaio" = no])
AC_SUBST(metaiolibs)
AC_SUBST(metaiocflags)
])

dnl======================================================================
dnl AC_GDS_CHECK_SASL
dnl   Check if SASL is installed
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_SASL],
[
have_sasl="no"
sasllibs=
saslcflags=
AC_MSG_CHECKING([for user specified sasl2 library])
AC_ARG_WITH(sasl, 
	    AC_HELP_STRING([--with-sasl@<:@=DIR@:>@],
			   [sasl2 library location @<:@SASL/lib@:>@]
			   ),
	    [have_sasl=yes],
	    [have_sasl=no]
)
AC_MSG_RESULT(${have_sasl})
if test "${have_ssl}" = "yes"
then
    saslcflags="-I${with_sasl}/include -DHAVE_SASL=2"
    sasllibs="-L${with_sasl}/lib -lsasl2"
    have_sasl=${with_sasl}
else
    AC_CHECK_LIB(sasl2, sasl_client_new, have_sasl="yes", have_ssl="no")
    if test "${have_sasl}" != "no"
    then
        saslcflags=-DHAVE_SASL=2
        sasllibs=-lsasl2
	have_sasl=sasl2
    else
        AC_CHECK_LIB(sasl, sasl_client_new, have_sasl="yes", have_ssl="no")
        if test "${have_sasl}" = "yes"
        then
            saslcflags=-DHAVE_SASL=1
            sasllibs=-lsasl
	    have_sasl=${with_sasl}
        fi
    fi
fi
AC_SUBST(sasllibs)
AC_SUBST(saslcflags)
])

dnl======================================================================
dnl   Enable DTT compilation
dnl   
dnl======================================================================

AC_DEFUN([AC_GDS_ENABLE_DTT],
[
AC_MSG_CHECKING([whether to build dtt])
need_dtt=no
need_onlydtt=no
AC_ARG_ENABLE(dtt,
	      AC_HELP_STRING([--enable-dtt], [Build dtt]),
	      [need_dtt="yes"]
)
AC_ARG_ENABLE(only_dtt,
	      AC_HELP_STRING([--enable-only-dtt], [Build dtt only]),
	      [need_onlydtt="yes"]
)

if test $need_onlydtt = "yes"
then
    need_dtt="yes"
    AC_MSG_RESULT([Only])
else
    AC_MSG_RESULT([${need_dtt}])
fi

have_epics=
AC_ARG_WITH(
    [epics], 
    AC_HELP_STRING([--with-epics@<:@=DIR@:>@], 
                   [epics library location @<:@DTT/lib@:>@]
		   ),
    [have_epics=yes],
    [have_epics=no]
)

dtt_defines=
epicslib=
build_epics=no
if test $need_dtt = "yes"
then
    dtt_defines="-DLIGO_GDS -DTARGET=0"
    dtt_defines="$dtt_defines -D_CONFIG_DYNAMIC"

    arch=
    case $host in
    *solaris*) 
        dtt_defines="$dtt_defines -D__EXTENSIONS__"
	arch=solaris
	if test `uname -m` = "i86pc" ;
	then
	    arch=solaris_x86
	fi
	;;
    *linux*)
	dtt_defines="$dtt_defines -DPORTMAP"
	arch=linux
	if test `uname -m` = "x86_64" ;
	then
	    arch=linux_x86
	fi
	;;
    *)
	dtt_defines="$dtt_defines -DPORTMAP"
    esac

    if test "x$have_epics" = "xyes" -a "x$with_epics" != "xno" ; 
    then
        epicslib=""
    elif test "x$with_epics" = "x" -o "x$with_epics" = "xyes"; then
        AC_MSG_CHECKING([for epics archive for arch=$arch])
        if test -f "DTT/lib/libepics-${arch}.a" ;
	then
	    epicslib="-L../lib -lepics-${arch}"
	    build_epics=yes
	else
	    build_epics=no
	fi
	AC_MSG_RESULT([${build_epics}])
    else
	epicslib="-L${with_epics}/lib -lepics-$arch"
	build_epics=yes
    fi
fi

if test "x$build_epics" = "xno" ;
then
    dtt_defines="$dtt_defines -DGDS_NO_EPICS"
fi

AM_CONDITIONAL([build_dtt], [test "x$need_dtt" = "xyes"])
AM_CONDITIONAL([build_onlydtt], [test "x$need_onlydtt" = "xyes"])
AM_CONDITIONAL([build_ezca], [test "x$build_epics" = "xyes"])
AC_SUBST(dtt_defines)
AC_SUBST(epicslib)
])

dnl======================================================================
dnl GDS_BUILD_TYPE
dnl   Check build type: online, offline, ...
dnl======================================================================

AC_DEFUN([AC_GDS_BUILD_TYPE],
[
echo $ECHO_N "checking for GDSBUILD ... $ECHO_C"
buildtype="none"
AC_ARG_ENABLE(online,
	      AC_HELP_STRING([--enable-online], [Online program version]),
	      [buildtype="online"],
)
AC_ARG_ENABLE(offline,
	      AC_HELP_STRING([--enable-offline], [Offline program version]),
	      [buildtype="offline"],
)
if test "x$buildtype" = "xnone"
then
    buildtype="$GDSBUILD"
    case "x$buildtype" in
    xonline | xoffline)
      ;;
    *)
      echo "Online or offline build must be specified - Aborting"
      exit 1;
      ;;
    esac
fi

AM_CONDITIONAL([online], [ test "x$buildtype" = "xonline" ])
AM_CONDITIONAL([offline], [ test "x$buildtype" = "xoffline" ])
echo "$buildtype"

bld_dict="yes"
echo $ECHO_N "checking whether to build root_objects ... $ECHO_C"
AC_ARG_ENABLE(root_objects,
	      AC_HELP_STRING([--enable-root-objects], 
	         [Build root dictionary in shared objects]),
	      	 [bld_dict="yes"]
)

case "x$enable_root_objects" in
"x" | "xyes")
    if test $buildtype = online ; then
        bld_dict="yes"
    else
        bld_dict="no"
    fi
    ;;
*)
    bld_dict="no"
    ;;
esac
AM_CONDITIONAL([root_dict], [test "$bld_dict" = "yes"])
echo "$bld_dict"
])

dnl======================================================================
dnl GDS_CHECK_POSIX
dnl	
dnl======================================================================

AC_DEFUN([AC_GDS_CHECK_POSIX],
[
posixflags="-D_POSIX_C_SOURCE=199506L"
unix98flags="-D_XOPEN_SOURCE=500"
case "${host}" in
*linux*) unix98flags="-D_XOPEN_SOURCE=600" ;;
esac
])

dnl======================================================================
dnl AC_GDS_FINISH - Things to be done at the very end for GDS
dnl======================================================================
 
AC_DEFUN([AC_GDS_FINISH],
[
  dnl ===================================================================
  dnl  Setup $architecture
  dnl ===================================================================
architecture=
case "${host}" in
*linux*) architecture="linux" ;;
*solaris*) architecture="solaris" ;;
esac
AC_SUBST(architecture)
  dnl ===================================================================
  dnl  Setup $syslibs
  dnl ===================================================================
syslibs=
case "${host}" in
*linux*) syslibs="-lrt" ;;
*solaris*) syslibs="-lrt" ;;
esac
AC_SUBST(syslibs)
  dnl ===================================================================
  dnl  Setup $sysmtlibs
  dnl ===================================================================
sysmtlibs=
case "${host}" in
*linux*) sysmtlibs="-lrt -lpthread" ;;
*solaris*) sysmtlibs="-lrt -lpthread" ;;
esac
AC_SUBST(sysmtlibs)
  dnl ===================================================================
  dnl  Setup $netlib
  dnl ===================================================================
netlib=
case "${host}" in
*linux*) netlib="" ;;
*solaris*) netlib="-lnsl -lsocket" ;;
esac
AC_SUBST(netlib)
  dnl ===================================================================
  dnl  Setup $readlinelibs
  dnl ===================================================================
readlinelibs=
case "${host}" in
*linux*) readlinelibs="-lreadline -ltermcap" ;;
*solaris*) readlinelibs="-lreadline -ltermcap" ;;
esac
AC_SUBST(readlinelibs)
  dnl ===================================================================
  dnl  Specify warning message level
  dnl ===================================================================
case "$GDS_CC_FLAVOR" in
gnu)
  CFLAGS="-pipe -fPIC -Wall $posixflags $unix98flags"
  ;;
SunPRO)
  CFLAGS="-v -KPIC -xc99=none"
  ;;
esac
case "$GDS_CXX_FLAVOR" in
gnu)
  CXXFLAGS="-pipe -fPIC -Wall -Woverloaded-virtual $posixflags $unix98flags"
  ;;
SunPRO)
  CXXFLAGS="-KPIC"
  ;;
esac
  dnl ===================================================================
  dnl  Specify optimization
  dnl ===================================================================
opt="-O3"
hiopt="-O6"
noopt="-O0"
AC_SUBST(opt)
AC_SUBST(hiopt)
AC_SUBST(noopt)
  dnl ===================================================================
  dnl  Writing config files
  dnl ===================================================================
AC_CONFIG_FILES(config/PConfig.h:config/PConfig.in)
AC_CONFIG_FILES(External/Makefile.patch:External/Makefile.patch.in)
AC_CONFIG_FILES(Utilities/scripts/WatchDawg:Utilities/scripts/WatchDawg.in)
AC_CONFIG_FILES(Utilities/scripts/findProc:Utilities/scripts/findProc.in)
AC_CONFIG_FILES(Utilities/setup/setup:Utilities/setup/setup.in)
AC_CONFIG_FILES(Utilities/setup/setupb:Utilities/setup/setupb.in)
AC_CONFIG_FILES(Utilities/setup/dol-user-env.sh:Utilities/setup/dol-user-env.sh.in)
AC_CONFIG_FILES(Utilities/setup/dol-user-env.csh:Utilities/setup/dol-user-env.csh.in)
AC_CONFIG_FILES(Utilities/setup/dol.pc:Utilities/setup/dol.pc.in)
AC_CONFIG_FILES(Utilities/procmgt/mkTrendDirs:Utilities/procmgt/mkTrendDirs.in)
AC_CONFIG_FILES(Utilities/procmgt/procmgt:Utilities/procmgt/procmgt.in)
AC_CONFIG_FILES(Utilities/procmgt/procsetup:Utilities/procmgt/procsetup.in)
])
